/*
 * Copyright (c) 1997 - 1999 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#pragma ident	"@(#)rpc.getpeerinfo_server.c	7.4	99/11/10 SMI; TSOL 2.x"

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <arpa/inet.h>
#include <bsm/audit.h>
#include <netinet/in.h>
#include <tsix/t6attrs.h>
#include <tsol/priv.h>

#include "getpeerinfo.h"

extern int _debug;
extern void _msgout(char *msg);


peerinfo_res *
getpeerinfo_1(peerinfo_arg *arg, struct svc_req *req)
{
	static peerinfo_res	result;
	au_peerinfo_t		pinfo;
	priv_set_t		*pset;
	char			buf[80];
	struct netbuf		*netbuf;

	(void) memset(&result, 0, sizeof (peerinfo_res));

	pinfo.peer_pid = arg->pid;
	pinfo.peer_flags = PEER_GROUPINFO | PEER_MISCINFO;
	memcpy(&(pinfo.peergroupinfo), &(result.peerinfo_res_u.info.grpinfo),
	    sizeof (au_peergroupinfo_t));
	memcpy(&(pinfo.peermiscinfo), &(result.peerinfo_res_u.info.prinfo),
	    sizeof (au_peermiscinfo_t));

	/* get the privs */
	if ((pset = (priv_set_t *)t6get_attr(T6_PRIVILEGES,
	    req->rq_xprt->xp_tsol_incoming_attrsp)) == NULL) {
		result.rpcerrno = EINVAL;
		netbuf = svc_getrpccaller(req->rq_xprt);
		(void) sprintf(buf,
		    "unable to get request privileges for %d %s",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr));
		_msgout(buf);
		return (&result);
	}

	/* check that user has either PRIV_PROC_AUDIT_TCB */
	/* or PROC_AUDIT_APPL */
	if ((PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_TCB) == 0) &&
	    (PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_APPL) == 0)) {
		if (_debug) {
			netbuf = svc_getrpccaller(req->rq_xprt);
			(void) sprintf(buf,
			    "%d %s does not have appropriate audit privilege "
			    "only groups returned",
			    pinfo.peer_pid,
			    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->
			    sin_addr));
			_msgout(buf);
		}
		pinfo.peer_flags &= ~PEER_MISCINFO;
	}

	if (_debug) {
		netbuf = svc_getrpccaller(req->rq_xprt);
		(void) sprintf(buf, "processing call for %d, %s",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr));
		_msgout(buf);
	}

	if (auditon(A_GETPEERINFO, (caddr_t)&pinfo, 0) == -1) {
		result.rpcerrno = errno;
		netbuf = svc_getrpccaller(req->rq_xprt);
		(void) sprintf(buf, "%d %s auditon() call failed [error: %s]",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr),
		    strerror(errno));
		_msgout(buf);
		memcpy(&(result.peerinfo_res_u.info.grpinfo),
		    &(pinfo.peergroupinfo), sizeof (au_peergroupinfo_t));
		if (pinfo.peer_flags & PEER_MISCINFO) {
			memcpy(&(result.peerinfo_res_u.info.prinfo),
			    &(pinfo.peermiscinfo), sizeof (au_peermiscinfo_t));
		}
		return (&result);
	}

	memcpy(&(result.peerinfo_res_u.info.grpinfo), &(pinfo.peergroupinfo),
	    sizeof (au_peergroupinfo_t));
	memcpy(&(result.peerinfo_res_u.info.prinfo), &(pinfo.peermiscinfo),
	    sizeof (au_peermiscinfo_t));
	result.rpcerrno = 0;
	return (&result);
}


peerinfo_res *
getpeerinfo_2(peerinfo_arg2 *arg, struct svc_req *req)
{
	static peerinfo_res	result;
	au_peerinfo_t		pinfo;
	priv_set_t		*pset;
	char			buf[80];
	struct netbuf		*netbuf;

	(void) memset(&result, 0, sizeof (peerinfo_res));

	pinfo.peer_pid = arg->pid;
	pinfo.peer_flags = 0;

	if ((arg->type == ALLINFO) || (arg->type == GRPINFO)) {
		pinfo.peer_flags |= PEER_GROUPINFO;
		memcpy(&(pinfo.peergroupinfo),
		    &(result.peerinfo_res_u.info.grpinfo),
		    sizeof (au_peergroupinfo_t));
	}

	if ((arg->type == ALLINFO) || (arg->type == AUDINFO)) {
		pinfo.peer_flags |= PEER_MISCINFO;
		memcpy(&(pinfo.peermiscinfo),
		    &(result.peerinfo_res_u.info.prinfo),
		    sizeof (au_peermiscinfo_t));

		/* get the privs */
		if ((pset = (priv_set_t *)t6get_attr(T6_PRIVILEGES,
		    req->rq_xprt->xp_tsol_incoming_attrsp)) == NULL) {
			netbuf = svc_getrpccaller(req->rq_xprt);
			result.rpcerrno = EINVAL;
			(void) sprintf(buf,
			    "unable to get request privileges for %d %s",
			    pinfo.peer_pid,
			    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->
			    sin_addr));
			_msgout(buf);
			return (&result);
		}

		/* check that user has either PRIV_PROC_AUDIT_TCB */
		/* or PROC_AUDIT_APPL */
		if ((PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_TCB) == 0) &&
		    (PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_APPL) == 0)) {
			netbuf = svc_getrpccaller(req->rq_xprt);
			result.rpcerrno = EACCES;
			(void) sprintf(buf,
			    "%d %s does not have appropriate audit privilege",
			    pinfo.peer_pid,
			    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->
			    sin_addr));
			_msgout(buf);
			return (&result);
		}
	}

	if (_debug) {
		netbuf = svc_getrpccaller(req->rq_xprt);
		(void) sprintf(buf, "processing call for %d, %s",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr));
		_msgout(buf);
	}

	if (auditon(A_GETPEERINFO, (caddr_t)&pinfo, 0) == -1) {
		netbuf = svc_getrpccaller(req->rq_xprt);
		result.rpcerrno = errno;
		(void) sprintf(buf, "%d %s auditon() call failed [error: %s]",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr),
		    strerror(errno));
		_msgout(buf);
		if (pinfo.peer_flags & PEER_GROUPINFO) {
			memcpy(&(result.peerinfo_res_u.info.grpinfo),
			    &(pinfo.peergroupinfo),
			    sizeof (au_peergroupinfo_t));
		}
		if (pinfo.peer_flags & PEER_MISCINFO) {
			memcpy(&(result.peerinfo_res_u.info.prinfo),
			    &(pinfo.peermiscinfo), sizeof (au_peermiscinfo_t));
		}
		return (&result);
	}

	if (pinfo.peer_flags & PEER_GROUPINFO) {
		memcpy(&(result.peerinfo_res_u.info.grpinfo),
		    &(pinfo.peergroupinfo), sizeof (au_peergroupinfo_t));
	}
	if (pinfo.peer_flags & PEER_MISCINFO) {
		memcpy(&(result.peerinfo_res_u.info.prinfo),
		    &(pinfo.peermiscinfo), sizeof (au_peermiscinfo_t));
	}
	result.rpcerrno = 0;
	return (&result);
}
